        LIST p=16f1827

;==========================================================================
;  Jobeaux Processor Main Module
; 
;  Copyright (C) 2011
;	This program is free software: you can redistribute it and/or modify
;	it under the terms of the GNU General Public License as published by
;	the Free Software Foundation, either version 3 of the License, or
;	(at your option) any later version.
;
;	This program is distributed in the hope that it will be useful,
;	but WITHOUT ANY WARRANTY; without even the implied warranty of
;	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;	GNU General Public License for more details.
;
;	You should have received a copy of the GNU General Public License
;	along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
; Revision History:
;	Who		Date	Description
;	sgt		TBD		Initial Release
;==========================================================================

#include "jobeaux.inc"
		SUBTITLE "Main Module"
		__config _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _MCLRE_OFF
		__config _CONFIG2, _WRT_OFF & _PLLEN_OFF & _LVP_OFF

		EXTERN	InitMotor, InitHeater, InitTimer, InitDisplay
		EXTERN	InitTempDisplay, ConfigureUnits
		EXTERN	ReadSpeed, ReadTemp, ReadReset
		EXTERN	TendMotor, TendHeater, TendDisplay
		EXTERN	DisplaySplash, StartMotor
		EXTERN	TimerIntHandler, RotIntHandler
		EXTERN	msDelay

;==========================================================================
; RESET VECTOR
; Program execution starts at location 0 on reset
;==========================================================================
BOOT	CODE 0x0000
		pagesel Start
		goto Start

;==========================================================================
; INTERRUPT VECTOR
; Interrupt vector is at location 4
;==========================================================================
INTR	CODE 0x0004
		pagesel	IntHandler
		goto	IntHandler


;==========================================================================
; MAIN code body.
; This can be anywhere - linker decides.
;==========================================================================
MAIN	CODE

;==========================================================================
; Interrupt handler
; Hand off to timer and direction sensor interrupt handlers.
; It is the responsibility of each individual inteerupt routine to
; determine if its interrupt was actually requested.
;==========================================================================
IntHandler:
		pagesel TimerIntHandler
		call	TimerIntHandler
		pagesel	RotIntHandler
		call	RotIntHandler		
		retfie


;==========================================================================
; Main Program Code
; Initialize all devices then enter main polling loop
;==========================================================================
Start:
		pagesel InitWorld
		call InitWorld

		call CheckSetup				; Holding SW3 at power on allows setting F/C
		call DisplaySplash

		; Reinitialize the timer so it starts at 00:00:00 after splash screen
		pagesel	InitTimer
		call	InitTimer

		call	StartMotor

; This is where all the work gets done.
Loop:
		pagesel ReadSpeed
		call ReadSpeed

		pagesel ReadTemp
		call ReadTemp

		pagesel ReadReset
		call ReadReset

		pagesel TendMotor
		call TendMotor

		pagesel TendHeater
		call TendHeater

		pagesel TendDisplay
		call TendDisplay

		goto Loop


;==========================================================================
; Initialize the world on power up
;==========================================================================
InitWorld:
		call InitCPU

		pagesel InitDisplay
		call	InitDisplay

		pagesel	InitTempDisplay
		call	InitTempDisplay

		pagesel	InitTimer
		call	InitTimer

		pagesel InitHeater
		call	InitHeater

		pagesel InitMotor
		call	InitMotor

		return

;==========================================================================
; Initialize the CPU I/O ports and other settings
;
; Digital Inputs: RA6, RB0
; Digital Outputs: RA2, RA4, RB5
; Analog Inputs: AN0, AN11
; PWM Outputs: P1A
; ADC Voltage ref Input: VREF+
; SPI Interface (all outputs): RA1, RA7, SCK1, SDO1
; 32,768 KHz Timer1 clock: T1OSI, T1OSO
; Internal Processor clock speed: 4MHz
; Timer1 generates interrupt @ 1 second intervals
; Modifies: bank
;==========================================================================
InitCPU:
		; Set up I/O pins
		; Inputs/Outputs:
		banksel	TRISA				; Bank 1
		movlw	B'00111001'			; TRISA0 | TRISA3 | TRISA4 | TRISA5 inputs
		movwf	TRISA
		movlw	B'11000011'			; TRISB0 | TRISB1 | TRISB6 | TRISB7 inputs
		movwf	TRISB

		; Internal pullup on RA5 (Still Bank 1)
		bcf		OPTION_REG, NOT_WPUEN
		
		; Configure ADC (also on Bank 1)
		movlw	B'10010010'			; Right Justify, Fosc/8, Vref=VREF+/Vss
		movwf	ADCON1

		; Finish internal pullup configuration
		banksel	WPUA				; Bank 4
		bsf		WPUA, WPUA5			; RA5 ON
		movlw	B'00000000'			; RB* OFF
		movwf	WPUB

		; Analog inputs:
		banksel	ANSELA				; Bank 3
		movlw	B'00000001'			; ANSA0
		movwf	ANSELA
		movlw	B'00000010'			; ANSB1
		movwf	ANSELB

		; Enable 32KHz crystal
		banksel	T1CON				; Bank 0
		movlw	B'00001100'			; Enable LP oscillator, don't sync
		movwf	T1CON

		; Initialize outputs (also Bank 0)
		movlw	B'10000100'			; /CSB=1, RS=0, Direction=1, Brake=0
		movwf	PORTA
		movlw	B'00010010'			; Motor Ctl=0, Heater Ctl=1, SDO1=0, SCK1=1
		movwf	PORTB

		; Configure SPI
		banksel	SSP1CON1			; Bank 4
		movlw	B'00110000'			; Enable, Idle hi, 1MHz clock
		movwf	SSP1CON1

		; Configure rotation sensor interrupt detector
		banksel	IOCBP				; Bank 7
		movlw	B'00000001'			; Int on rising edge of RB0
		movwf	IOCBP
		movwf	IOCBN				; Also int on falling edge of RB0

		; Select CPU clock speed by setting IRCF<3:0> bits of OSCCON
		banksel	OSCCON				; Bank 1
		movlw	B'01101010'			; 4MHz, int osc
		movwf	OSCCON

		; Wait for HF osc to be ready
		btfss	OSCSTAT, HFIOFR
		goto	$-1

		; Give the LP oscillator 40 msec to start
		movlw	40
		pagesel	msDelay
		call	msDelay

		; Set up Timer2 for PWM operation
		banksel	T2CON				; Bank 0
		movlw	B'00000001'			; Prescaler is 4
		movwf	T2CON
		movlw	PWM_PERIOD
		movwf	PR2					; Set PWM period
		
		; Set CCP1 configuration for PWM operation
		banksel	CCP1CON				; Bank 5
		movlw	B'00001100'			; Single output, PWM Mode, active hi
		movwf	CCP1CON
		movlw	B'00000001'			; Output goes to P1A
		movwf	PSTR1CON

		; Set up Timer1
		; Make sure there's no current int request -
		; Clear TMR1IF in PIR1 register
		banksel	PIR1				; Bank 0
		bcf 	PIR1, TMR1IF

		; Is LP oscillator ready?
		btfss	OSCSTAT, T1OSCR
		goto	useIntClock			; No.  Use internal clock for timer

		movlw	B'10001100'			; Source is oscillator, 1:1
		movwf	T1CON

		; Initialize Timer1 for 32.768 KHz clock
		; Same bank as PIR1
		movlw	T1_LP_START_HI
		movwf	TMR1H
		movlw	T1_LP_START_LO
		movwf	TMR1L
		goto	enableT1

useIntClock:
		; Oscillator not working for some reason.  Use instruction clock
		; as Timer1 clock.  This will be less accurate than the crystal...
		movlw	B'00110000'			; Source is instr clock, 1:8, LP osc off
		movwf	T1CON

		; Initialize timer for 125KHz clock
		movlw	T1_IC_START_HI
		movwf	TMR1H
		movlw	T1_IC_START_LO
		movwf	TMR1L

enableT1:
		; Now enable Timer1 interrupt by
		; setting TMR1IE in PIE1 register
		banksel	PIE1				; Bank 1
		bsf		PIE1, TMR1IE

		; Enable interrupts
		movlw	B'11001000'			; IOCIE, PEIE, GIE
		movwf	INTCON

		; Turn Timer1 & Timer2 on
		banksel	T1CON				; Bank 0
		bsf		T1CON, TMR1ON
		clrf	T1GCON
		bsf		T2CON, TMR2ON

		return

;==========================================================================
; Check if SW3 is pressed.
; This funtion is called just before the splash screen is displayed, so
; if SW3 is held at power on, we will see it here.
; Modifies: bank, page
;==========================================================================
CheckSetup:
		banksel	TIMER_RESET_PORT	; Bank 0
		btfsc	TIMER_RESET_PORT, TIMER_RESET_IN
		return						; Not pressed

		; Button is pressed!
		pagesel	ConfigureUnits
		call	ConfigureUnits
		return

	end
