;==========================================================================
;  Jobeaux Processor Motor Control Module
; 
;  Copyright (C) 2011
;	This program is free software: you can redistribute it and/or modify
;	it under the terms of the GNU General Public License as published by
;	the Free Software Foundation, either version 3 of the License, or
;	(at your option) any later version.
;
;	This program is distributed in the hope that it will be useful,
;	but WITHOUT ANY WARRANTY; without even the implied warranty of
;	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;	GNU General Public License for more details.
;
;	You should have received a copy of the GNU General Public License
;	along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
; Revision History:
;	Who		Date	Description
;	sgt		TBD		Initial Release
;==========================================================================

#include "jobeaux.inc"
	SUBTITLE "Motor Control Module"

	GLOBAL	InitMotor, ReadSpeed, StartMotor, TendMotor
	GLOBAL	RotIntHandler
	GLOBAL	StopTicks
	GLOBAL	Rotate, MotorState, Phase		; DEBUG

	UDATA_SHR
Rotate		RES	1					; Rotation counter
Phase		RES	1					; Tracks rising/falling edges
StopTicks	RES	1					; Tick counter for stopping motor

	UDATA
Speed		RES	1					; Motor speed. 1 = high, 0 = low
MotorState	RES	1					; Current motor state
Direction	RES	1					; Current motor direction (0/1)

MOTOR	CODE

;==========================================================================
; Initialize the motor controller
; Modifies: bank
;==========================================================================
InitMotor:
		movlw	REVERSE_COUNT
		movwf	Rotate				; Init rotation counter
		clrf	StopTicks
		clrf	Phase
		banksel	Direction
		clrf	Direction			; ...and motor direction
		banksel	MotorState
		movlw	MSTATE_STOPPED		; ...and set state to stopped
		movwf	MotorState
		call	ReadSpeed			; Get initial speed setting

		; Set initial state for DRV8801:
		; Enable=0, Phase=0, {Mode1=1, Mode2=0} (=> Brake=1)
		clrw
		banksel	CCPR1L				; Bank 5
		movwf	CCPR1L
		banksel	MOTOR_DIR_PORT		; Bank 0
		bcf		MOTOR_DIR_PORT, MOTOR_DIR_OUT
		bsf		MOTOR_BRAKE_PORT, MOTOR_BRAKE_OUT

		return


;==========================================================================
; Read the setting of the motor speed switch (on RA6) & save value for
; later reference.
; Modifies: bank
;==========================================================================
ReadSpeed:
		clrw
		banksel	MOTOR_SPEED_PORT	; Bank 0
		btfsc	MOTOR_SPEED_PORT, MOTOR_SPEED_IN
		incf	WREG, W
		banksel	Speed
		movwf	Speed

		return


;==========================================================================
; Tend the motor based on current MotorState
; Modifies: bank
;==========================================================================
TendMotor:
		banksel	MotorState
		movfw	MotorState
		brw
		goto	MotorStopped		; MSTATE_STOPPED (0)
		goto	MotorStarting		; MSTATE_STARTING (1)
		goto	MotorRunning		; MSTATE_TURNING (2)
		goto	MotorStopping		; MSTATE_STOPPING (3)


;==========================================================================
; MSTATE_STOPPED handler
; The motor is stopped (brake applied).  We wait for some amount of time
; (STOP_TIME timer ticks) and then start it running again.
; Modifies: bank
;==========================================================================
MotorStopped:
		movf	StopTicks, F
		btfss	STATUS, Z			; Has the delay time elapsed?
		return						; No - try again later

		; We have waited for the prescribed time.  We can start the motor
		; again after reversing the direction value.
		clrw						; Assume we want to set Direction to 0
		banksel	Direction
		movf	Direction, F
		btfsc	STATUS, Z
		incf	WREG, W				; Was zero - now make it 1
		movwf	Direction			; Save updated value

		call	StartMotor
		return


;==========================================================================
; MSTATE_STARTING handler
; The motor is starting (brake applied).  We wait until we see a falling
; edge from the rotation sensor and then turn off the brake and consider
; it running.
; Modifies: bank
;==========================================================================
MotorStarting:
		; FixFixFix - If we don't start in a reasonable time, shut down
		; motor and indicate fault?
		btfss	Phase, PHASE_FALLING	; Have we seen a falling edge?
		return							; No.  Keep waiting.

		; Falling edge detected - the motor is in fact running.
		; Initialize rotation counter
		movlw	REVERSE_COUNT
		movwf	Rotate

		; And set to TURNING state
		banksel	MotorState
		movlw	MSTATE_TURNING
		movwf	MotorState
		return


;==========================================================================
; MSTATE_TURNING handler
; The motor is running.  We let it run for some number of full revolutions
; (specified by REVERSE_COUNT in jobeaux.inc) and then go to the STOPPING
; state.
; This code assumes a single rotation sensor pulse per revolution.
; Modifies: bank
;==========================================================================
MotorRunning:
		movf	Rotate, F
		btfss	STATUS, Z				; Have we reached the stop point?
		return							; Nope.

		; We've done two rotations - move to STOPPING state
		; Stop PWM1
		clrw
		banksel	CCPR1L					; Bank 5
		movwf	CCPR1L
		movlw	COAST_TIME
		movwf	StopTicks

		; Turn off the brake - let it coast initially
		banksel	MOTOR_BRAKE_PORT
		bcf		MOTOR_BRAKE_PORT, MOTOR_BRAKE_OUT

		banksel	MotorState
		movlw	MSTATE_STOPPING
		movwf	MotorState
		return


;==========================================================================
; MSTATE_STOPPING handler
; The motor is coasting (brake not applied).  We let it coast for some
; amount of time (COAST_TIME timer ticks) and then hit the brakes.
; Modifies: bank
;==========================================================================
MotorStopping:
		movf	StopTicks, F
		btfss	STATUS, Z				; Has the delay time elapsed?
		return							; No - try again later

		; We have waited for the prescribed time.  We can now hit the brakes.
		banksel	MOTOR_BRAKE_PORT
		bsf		MOTOR_BRAKE_PORT, MOTOR_BRAKE_OUT

		; Update StopTicks for STOPPED state delay
		movlw	STOP_TIME
		movwf	StopTicks

		; And set new state
		banksel	MotorState
		movlw	MSTATE_STOPPED
		movwf	MotorState
		return


;==========================================================================
; Start the motor running.
; Brake should be set (MotorState = STOPPED) when calling this routine -
; Despite what the data sheet might indicate, the DRV8801 doesn't like to
; start the motor if brake not set.  This extends to the case where we're
; sending a PWM signal to the Enable input.  Therefore, brake is ON
; whenever we're starting or running.
; Modifies: bank
;==========================================================================
StartMotor:
		clrf	Phase

		; Update the motor state
		banksel	MotorState
		movlw	MSTATE_STARTING
		movwf	MotorState

		; Set the direction
		banksel	Direction
		movf	Direction, F
		banksel	MOTOR_DIR_PORT
		btfsc	STATUS, Z
		goto	dir0
		bsf		MOTOR_DIR_PORT, MOTOR_DIR_OUT
		goto	enableMotor

dir0:	bcf		MOTOR_DIR_PORT, MOTOR_DIR_OUT

enableMotor:
		; Set PWM1 to run motor at selected speed
		; Set duty cycle based on switch setting
		movlw	PWM_DUTY_SLOW			; Assume low speed
		banksel	Speed
		movf	Speed, F
		btfss	STATUS, Z
		movlw	PWM_DUTY_FAST			; Nope, it's high speed
		banksel	CCPR1L					; Bank 5
		movwf	CCPR1L

		return


;==========================================================================
; Rotation sensor interrupt handler
; Just counts the number of motor rotations - it's up to the TendMotor
; function to figure out when to reverse rotation and reset the counter...
; Phase is used to debounce the input signal.
; Modifies: bank
;==========================================================================
RotIntHandler:
		banksel	IOCBF				; Bank 7
		btfss	IOCBF, IOCBF0		; See if we actually got an interrupt
		return						; No - just ignore

		; Yes - we saw a rotation event
		bcf		IOCBF, IOCBF0		; Clear the int request
		banksel	MOTOR_ROT_PORT		; Determine if rising or falling
		btfss	MOTOR_ROT_PORT, MOTOR_ROT_IN
		goto	fell

		; Detected rising edge
		btfsc	Phase, PHASE_RISING	; Did we already see a rising edge?
		return						; Yes - can't have another until we
									; first see a falling edge
		movf	Rotate, F
		btfss	STATUS, Z			; If not already at zero,
		decf	Rotate				; bump the counter
		bsf		Phase, PHASE_RISING	; and update Phase
		bcf		Phase, PHASE_FALLING
		return

fell:
		; Detected falling edge
		btfsc	Phase, PHASE_FALLING
		return

		bcf		Phase, PHASE_RISING	; Update Phase
		bsf		Phase, PHASE_FALLING
		return

	end
