;==========================================================================
;  Jobeaux Processor Timer Module
; 
;  Copyright (C) 2011
;	This program is free software: you can redistribute it and/or modify
;	it under the terms of the GNU General Public License as published by
;	the Free Software Foundation, either version 3 of the License, or
;	(at your option) any later version.
;
;	This program is distributed in the hope that it will be useful,
;	but WITHOUT ANY WARRANTY; without even the implied warranty of
;	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;	GNU General Public License for more details.
;
;	You should have received a copy of the GNU General Public License
;	along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
; Revision History:
;	Who		Date	Description
;	sgt		TBD		Initial Release
;==========================================================================

#include "jobeaux.inc"
	SUBTITLE "Timer Module"

	GLOBAL	InitTimer, ReadReset, TimerIntHandler
	GLOBAL	DisplayTimer, OKUpdate

	EXTERN	StopTicks

MAX_SEC		EQU	0x60
MAX_MIN		EQU	0x60
MAX_HRS		EQU	0xA0
BCD_BUMP	EQU	7

;==========================================================================
; The clock is stored as a BCD value in 3 bytes (HH:MM:SS)
; This makes generating the display a little easier
; (no divide by 10), but makes incrementing the value in the ISR
; a little more involved.
; These values are stored in shared RAM (Common RAM), and that helps
; reduce the overhead in the interrupt handler a little
; (saves a banksel)
;==========================================================================
	UDATA_SHR
Hours		RES 1
Minutes		RES 1
Seconds		RES 1
Maxed		RES 1
Ticker		RES	1					; Fractional second counter
OKUpdate	RES	1


TIMER	CODE

;==========================================================================
; Initialize the timer to 00:00:00
; Modifies: bank
;==========================================================================
InitTimer:
		bcf		INTCON, GIE			; Disable Interrupts
		clrf	Hours
		clrf	Minutes
		clrf	Seconds
		clrf	Maxed
		movlw	TICKS_PER_SEC
		movwf	Ticker

		; Reset the counter
		banksel	T1CON				; Bank 0
		bcf		T1CON, TMR1ON		; Stop Timer1
		btfss	T1CON, TMR1CS1		; Which clock is running?
		goto	instClockInit		; Internal

		; Running on LP clock.
		movlw	T1_LP_START_HI		; Set to initial value
		movwf	TMR1H
		movlw	T1_LP_START_LO
		movwf	TMR1L
		goto	restartTimer

instClockInit:
		; Instruction clock/8
		movlw	T1_IC_START_HI
		movwf	TMR1H
		movlw	T1_IC_START_LO
		movwf	TMR1L

restartTimer:
		bsf		T1CON, TMR1ON		; Restart Timer1
		bsf		INTCON, GIE			; Enable Interrupts

		return


;==========================================================================
; Read the setting of the timer reset switch.
; Timer reset switch is on RA4
; If it is pressed, reset the timer value to 00:00:00
; Modifies: bank, page
;==========================================================================
ReadReset:
		banksel	TIMER_RESET_PORT	; Bank 0
		btfsc	TIMER_RESET_PORT, TIMER_RESET_IN
		return

		pagesel	InitTimer			; Button is pressed!
		call	InitTimer

		movlw	1
		movwf	OKUpdate			; Update the display next opportunity

		return


;==========================================================================
; Process a Timer1 interrupt
; These occur once a second.
; The timer value is incremented and the interrupt is cleared.
; Modifies: bank
;==========================================================================
TimerIntHandler:
		; Verify Timer1 requesting interrupt
		banksel	PIR1				; Bank 0
		btfss	PIR1, TMR1IF
		return						; Not set - must be someone else

		movlw	1
		movwf	OKUpdate			; Update the display next opportunity

		; Clear the interrupt request (TMR1IF in PIR1 register)
		bcf 	PIR1, TMR1IF

		btfss	T1CON, TMR1CS1		; Which clock is running?
		goto	instClockHandler	; Internal

		; LP Clock -
		; Reset Timer1 to initial value
		; Same bank as PIR1
		movlw	T1_LP_START_HI
		movwf	TMR1H
		movlw	T1_LP_START_LO
		addwf	TMR1L, F
		goto	checkClock

instClockHandler:
		; Running on internal clock.
		; Reset the counter
		movlw	T1_IC_START_HI
		movwf	TMR1H
		; Due to interrupt overhead and fast clock, low byte may have
		; incremented already, throwing the timing off.  This gets us
		; closer...
		movlw	T1_IC_START_LO
		addwf	TMR1L, F

checkClock:
		movf	StopTicks, F
		btfss	STATUS, Z			; If the motor is waiting
		decf	StopTicks, F		; tell it the clock ticked

		decfsz	Ticker, F			; Count the ticks til we get to 1 sec
		return

		movlw	TICKS_PER_SEC		; A second has elapsed! Reset counter
		movwf	Ticker

		; Is the clock maxed out?
		btfsc	Maxed, 1
		return						; Yes - Just leave it there

		; Increment the clock (BCD values)
		movfw	Seconds
		andlw	0x0F
		sublw	0x09				; Low nibble @ 9?
		btfsc	STATUS, Z
		goto 	secwrap
		incf	Seconds, F			; No, just increment
		return

secwrap:
		movfw	Seconds
		addlw	BCD_BUMP			; (9+7) = 16: low nibble = 0, hi += 1
		movwf	Seconds
		sublw	MAX_SEC				; Have we hit the next minute?
		btfss	STATUS, Z
		return						; No. Done.

		clrf	Seconds				; Yes. Clear seconds & inc minutes
		movfw	Minutes
		andlw	0x0F
		sublw	0x09				; Low nibble @ 9?
		btfsc	STATUS, Z
		goto 	minwrap
		incf	Minutes, F			; No, just increment
		return

minwrap:
		movfw	Minutes
		addlw	BCD_BUMP			; (9+7) = 16: low nibble = 0, hi += 1
		movwf	Minutes
		sublw	MAX_MIN				; Have we hit the next hour?
		btfss	STATUS, Z
		return						; No. Done.
		clrf	Minutes				; Yes. Clear minutes & inc hours

		movfw	Hours
		andlw	0x0F
		sublw	0x09				; Low nibble @ 9?
		btfsc	STATUS, Z
		goto 	hrwrap
		incf	Hours, F			; No, just increment
		return

hrwrap:
		movfw	Hours
		addlw	BCD_BUMP			; (9+7) = 16: low nibble = 0, hi += 1
		movwf	Hours
		sublw	MAX_HRS				; Have we maxed out?
		btfsc	STATUS, Z
		bsf		Maxed, 1			; Yes. Set flag
		return						; Done.


;==========================================================================
; Display the current timer value
; Actually just returns the character for the current column (in W)
; and the calling routine does the output
;==========================================================================
DisplayTimer:
		brw
		retlw	' '					; Column 0
		retlw	' '					; Column 1
		retlw	' '					; Column 2
		retlw	' '					; Column 3
		goto	htens				; Column 4: Hours tens digit
		goto	hones				; Column 5: Hours ones digit
		retlw	':'					; Column 6: Separator
		goto	mtens				; Column 7: Minutes tens digit
		goto	mones				; Column 8: Minutes ones digit
		retlw	':'					; Column 9: Separator
		goto	stens				; Column 10: Seconds tens digit
		goto	sones				; Column 11: Seconds ones digit
		retlw	' '					; Column 12
		goto	timerIndicator		; Column 13
		retlw	' '					; Column 14
		retlw	' '					; Column 15

htens:
		; Is the clock maxed out?
		btfsc	Maxed, 1
		retlw	'*'					; Yes - Display *

		movfw	Hours				; Get current hours value
tensdigit:
		swapf	WREG, W				; Put the tens digit in the low nibble
onesdigit:
		andlw	0x0F				; Mask it
		addlw	'0'					; and make it ASCII
		return

hones:
		; Is the clock maxed out?
		btfsc	Maxed, 1
		retlw	'*'					; Yes - Display *

		movfw	Hours
		goto	onesdigit

mtens:
		; Is the clock maxed out?
		btfsc	Maxed, 1
		retlw	'*'					; Yes - Display *

		movfw	Minutes
		goto	tensdigit

mones:
		; Is the clock maxed out?
		btfsc	Maxed, 1
		retlw	'*'					; Yes - Display *

		movfw	Minutes
		goto	onesdigit

stens:
		; Is the clock maxed out?
		btfsc	Maxed, 1
		retlw	'*'					; Yes - Display *

		movfw	Seconds
		goto	tensdigit

sones:
		; Is the clock maxed out?
		btfsc	Maxed, 1
		retlw	'*'					; Yes - Display *

		movfw	Seconds
		goto	onesdigit

timerIndicator:
		; Indicate if we're running on the internal timer so the user knows
		; it may not be as accurate as he wants
		banksel	T1CON
		btfss	T1CON, TMR1CS1		; Which clock is running?
		retlw	INTERNAL_CLOCK_IND	; Internal
		retlw	' '					; LP crystal oscillator

	end
